@@ -354,6 +354,22 @@ private:
 
 OPENDHT_PUBLIC extern const HexMap hex_map;
 
+inline std::string
+toHex(const uint8_t* data, size_t size) {
+    std::string ret(size * 2, '\0');
+    for (size_t i=0; i<size; i++) {
+        auto b = ret.data()+i*2;
+        const auto& m = hex_map[data[i]];
+        *((uint16_t*)b) = *((uint16_t*)&m);
+    }
+    return ret;
+}
+
+inline std::string
+toHex(const std::vector<uint8_t>& data) {
+    return toHex(data.data(), data.size());
+}
+
 template <size_t N>
 const char*
 Hash<N>::to_c_str() const
